local enable_debug_prints = false
-------------------------------------------------------------------------------
-- Faction adding code. Don't touch this unless you know what your doing. :) --
-------------------------------------------------------------------------------
local function debug_log(...)
	if enable_debug_prints then
		log(...)
	end
end

local function string_list_to_idstring_list(list)
	local output = {}
	for key, value in pairs(list) do
		output[key] = Idstring(value)
	end
	return output
end

local function add_faction(faction_name, chatter, prefixes, groups, difficulty_specific)
	debug_log("[Faction Adder] Adding Faction: " .. faction_name)
	LevelsTweakData.LevelType[faction_name] = faction_name
	-- BeardLib cares about this. Even though they're all just glorified paths to strings. :sader:
	Hooks:PostHook(LevelsTweakData, "init", "unique_special_level_group_hook_" .. faction_name, function(self)
		self.ai_groups[faction_name] = faction_name
	end)

	-- Setup unit category hook.
	Hooks:PostHook(GroupAITweakData, "_init_unit_categories", "unique_special_faction_hook_" .. faction_name, function(groupai_td, difficulty_index)
		-- Get difficulty specific unit lists.
		local difficulty_specific_groups = difficulty_specific[difficulty_index]
		if difficulty_specific_groups then
			debug_log("[Faction Adder] Found Difficulty Specific: " .. tostring(difficulty_index))
			for category_name, unit_list in pairs(difficulty_specific_groups) do
				groups[category_name] = unit_list
			end
		end

		-- Add our faction group variants.
		for category_name, category_data in pairs(groupai_td.unit_categories) do
			local unit_list = groups[category_name]

			if unit_list then
				debug_log("[Faction Adder] Custom Group: " .. category_name)
				category_data.unit_types[faction_name] = string_list_to_idstring_list(unit_list)
			elseif category_data.unit_types.america then
				debug_log("[Faction Adder] No Custom Group: " .. category_name)
				category_data.unit_types[faction_name] = category_data.unit_types.america
			end
		end
	end)

	-- Setup the function used to set prefixes and chatter.
	CharacterTweakData["_init_region_" .. faction_name] = function(self)
		self:_init_region_america()

		self._default_chatter = chatter or self._default_chatter
		for key, value in pairs(prefixes) do
			self._unit_prefixes[key] = value
		end
	end
end

--------------------------
-- Faction Setup Stuff --
--------------------------

-- Define the factions internal name.
local faction_name = "response_type_hell"

-- Define voice data.
local chatter = "dispatch_generic_message"
local prefixes = {
	cop = "l",
	swat = "l",
	heavy_swat = "l",
	taser = "tsr",
	cloaker = "clk",
	bulldozer = "bdz",
	medic = "mdc"
}

-- Define generic units.
-- If your really cool you can add groups from rebalances here as well and it'll handle that.
-- If you don't specify a group it'll just default to a copy of `america` so that shit doesn't explode.
local groups = {
	CS_cop_C45_R870 = {
		"units/matthelzor/characters/ene_imp_light/ene_imp_light"
	},
	CS_cop_stealth_MP5 = {
		"units/matthelzor/characters/ene_imp_light/ene_imp_light"
	},
	CS_swat_MP5 = {
		"units/matthelzor/characters/ene_imp_light/ene_imp_light"
	},
	CS_swat_R870 = {
		"units/matthelzor/characters/ene_imp_light/ene_imp_light"
	},
	CS_heavy_M4 = {
		"units/matthelzor/characters/ene_imp_heavy/ene_imp_heavy"
	},
	CS_heavy_R870  = {
		"units/matthelzor/characters/ene_imp_heavy/ene_imp_heavy"
	},
	CS_heavy_M4_w = {
		"units/matthelzor/characters/ene_imp_heavy/ene_imp_heavy"
	},
	FBI_swat_M4 = {
		"units/matthelzor/characters/ene_imp_light/ene_imp_light"
	},
	FBI_swat_R870 = {
		"units/matthelzor/characters/ene_imp_light/ene_imp_light"
	},
	FBI_heavy_G36 = {
		"units/matthelzor/characters/ene_imp_heavy/ene_imp_heavy"
	},
	FBI_heavy_R870  = {
		"units/matthelzor/characters/ene_imp_heavy/ene_imp_heavy"
	},
	FBI_heavy_G36_w = {
		"units/matthelzor/characters/ene_imp_heavy/ene_imp_heavy"
	},
}

-- Define difficulty specific units.
local difficulty_specific = {}

-- Add the faction.
add_faction(faction_name, chatter, prefixes, groups, difficulty_specific)