local level_id = Global.level_data and Global.level_data.level_id or ""
local difficulty = Global.game_settings and Global.game_settings.difficulty or "normal"

local enable_debug_prints = false
-------------------------------------------------------------------------------
-- Faction adding code. Don't touch this unless you know what your doing. :) --
-------------------------------------------------------------------------------
local function debug_log(...)
	if enable_debug_prints then
		log(...)
	end
end

local function string_list_to_idstring_list(list)
	local output = {}
	for key, value in pairs(list) do
		output[key] = Idstring(value)
	end
	return output
end

local function add_faction(faction_name, chatter, prefixes, groups, difficulty_specific)
	debug_log("[Faction Adder] Adding Faction: " .. faction_name)
	LevelsTweakData.LevelType[faction_name] = faction_name
	-- BeardLib cares about this. Even though they're all just glorified paths to strings. :sader:
	Hooks:PostHook(LevelsTweakData, "init", "unique_special_level_group_hook_" .. faction_name, function(self)
		self.ai_groups[faction_name] = faction_name
	end)

	-- Setup unit category hook.
	Hooks:PostHook(GroupAITweakData, "_init_unit_categories", "unique_special_faction_hook_" .. faction_name, function(groupai_td, difficulty_index)
		-- Get difficulty specific unit lists.
		local difficulty_specific_groups = difficulty_specific[difficulty_index]
		if difficulty_specific_groups then
			debug_log("[Faction Adder] Found Difficulty Specific: " .. tostring(difficulty_index))
			for category_name, unit_list in pairs(difficulty_specific_groups) do
				groups[category_name] = unit_list
			end
		end

		-- Add our faction group variants.
		for category_name, category_data in pairs(groupai_td.unit_categories) do
			local unit_list = groups[category_name]

			if unit_list then
				debug_log("[Faction Adder] Custom Group: " .. category_name)
				category_data.unit_types[faction_name] = string_list_to_idstring_list(unit_list)
			elseif category_data.unit_types.america then
				debug_log("[Faction Adder] No Custom Group: " .. category_name)
				category_data.unit_types[faction_name] = category_data.unit_types.america
			end
		end
	end)

	-- Setup the function used to set prefixes and chatter.
	CharacterTweakData["_init_region_" .. faction_name] = function(self)
		self:_init_region_america()

		self._default_chatter = chatter or self._default_chatter
		for key, value in pairs(prefixes) do
			self._unit_prefixes[key] = value
		end
	end
end

local function isDifficultyHigh()
	local high_difficulty = difficulty == "overkill_145" or difficulty == "easy_wish" or difficulty == "overkill_290" or difficulty == "sm_wish"
	return high_difficulty
end

--------------------------
-- Faction Setup Stuff --
--------------------------

-- Define the factions internal name.
local faction_name = "response_type_swat"

-- Define voice data.
local chatter = "dispatch_generic_message"
local prefixes = {
	cop = "l",
	swat = "l",
	heavy_swat = "l",
	taser = "tsr",
	cloaker = "clk",
	bulldozer = "bdz",
	medic = "mdc"
}

-- Define generic units.
-- If your really cool you can add groups from rebalances here as well and it'll handle that.
-- If you don't specify a group it'll just default to a copy of `america` so that shit doesn't explode.
local groups = {
	spooc = {
		"units/payday2/characters/ene_spook_1/ene_spook_1"
	},
	CS_cop_stealth_MP5 = {
		"units/payday2/characters/ene_swat_1/ene_swat_1"
	},
	CS_cop_C45_R870 = {
		"units/payday2/characters/ene_swat_1/ene_swat_1",
		"units/payday2/characters/ene_swat_2/ene_swat_2"
	},
	CS_swat_MP5 = {
		"units/payday2/characters/ene_swat_1/ene_swat_1"
	},
	CS_swat_R870 = {
		"units/payday2/characters/ene_swat_2/ene_swat_2"
	},
	CS_heavy_M4 = {
		"units/payday2/characters/ene_swat_heavy_1/ene_swat_heavy_1"
	},
	CS_heavy_R870  = {
		"units/payday2/characters/ene_swat_heavy_r870/ene_swat_heavy_r870"
	},
	CS_heavy_M4_w = {
		"units/payday2/characters/ene_swat_heavy_1/ene_swat_heavy_1"
	},
	CS_tazer = {
		"units/payday2/characters/ene_tazer_1/ene_tazer_1"
	},
	CS_shield = {
		"units/payday2/characters/ene_shield_2/ene_shield_2"
	},
	FBI_suit_C45_M4 = {
		"units/payday2/characters/ene_swat_1/ene_swat_1",
		"units/payday2/characters/ene_swat_2/ene_swat_2"
	},
	FBI_suit_M4_MP5 = {
		"units/payday2/characters/ene_swat_heavy_1/ene_swat_heavy_1",
		"units/payday2/characters/ene_swat_1/ene_swat_1"
	},
	FBI_suit_stealth_MP5 = {
		"units/payday2/characters/ene_swat_1/ene_swat_1"
	},
	FBI_swat_M4 = {
		"units/payday2/characters/ene_swat_1/ene_swat_1"
	},
	FBI_swat_R870 = {
		"units/payday2/characters/ene_swat_2/ene_swat_2"
	},
	FBI_heavy_G36 = {
		"units/payday2/characters/ene_swat_heavy_1/ene_swat_heavy_1"
	},
	FBI_heavy_R870  = {
		"units/payday2/characters/ene_swat_heavy_r870/ene_swat_heavy_r870"
	},
	FBI_heavy_G36_w = {
		"units/payday2/characters/ene_swat_heavy_1/ene_swat_heavy_1"
	},
	FBI_shield = {
		"units/payday2/characters/ene_shield_2/ene_shield_2"
	},
	FBI_tank = {
		"units/payday2/characters/ene_bulldozer_2/ene_bulldozer_2"
	},
	marshal_marksman = {
		"units/payday2/characters/ene_sniper_1/ene_sniper_1"
	},
	marshal_shield = {
		"units/payday2/characters/ene_shield_2/ene_shield_2"
	},
}

-- Define difficulty specific units.
local difficulty_specific = {}

-- Define level specific units.
if level_id == "chas" or level_id == "sand" or level_id == "pent" then
	if level_id ~= "pent" and not isDifficultyHigh() then
		table.insert(groups.CS_swat_MP5, "units/pd2_dlc_chas/characters/ene_male_chas_police_01/ene_male_chas_police_01")
		table.insert(groups.CS_swat_MP5, "units/pd2_dlc_chas/characters/ene_male_chas_police_02/ene_male_chas_police_02")
	end
	
	groups.CS_cop_C45_R870 = {}
	table.insert(groups.CS_cop_C45_R870, "units/pd2_dlc_chas/characters/ene_male_chas_police_01/ene_male_chas_police_01")
	table.insert(groups.CS_cop_C45_R870, "units/pd2_dlc_chas/characters/ene_male_chas_police_02/ene_male_chas_police_02")
elseif level_id == "ranc" or level_id == "corp" then
	if level_id ~= "corp" and not isDifficultyHigh() then
		table.insert(groups.CS_swat_MP5, "units/pd2_dlc_ranc/characters/ene_male_ranc_ranger_01/ene_male_ranc_ranger_01")
		table.insert(groups.CS_swat_MP5, "units/pd2_dlc_chas/characters/ene_male_ranc_ranger_02/ene_male_ranc_ranger_02")
	end
	
	groups.CS_cop_C45_R870 = {}
	table.insert(groups.CS_cop_C45_R870, "units/pd2_dlc_ranc/characters/ene_male_ranc_ranger_01/ene_male_ranc_ranger_01")
	table.insert(groups.CS_cop_C45_R870, "units/pd2_dlc_ranc/characters/ene_male_ranc_ranger_02/ene_male_ranc_ranger_02")
elseif level_id == "rvd1" or level_id == "rvd2" then

	table.insert(groups.CS_swat_R870, "units/pd2_dlc_rvd/characters/ene_la_cop_3/ene_la_cop_3")
	table.insert(groups.CS_swat_MP5, "units/pd2_dlc_rvd/characters/ene_la_cop_4/ene_la_cop_4")

	groups.CS_cop_C45_R870 = {}
	table.insert(groups.CS_cop_C45_R870, "units/pd2_dlc_rvd/characters/ene_la_cop_1/ene_la_cop_1")
	table.insert(groups.CS_cop_C45_R870, "units/pd2_dlc_rvd/characters/ene_la_cop_2/ene_la_cop_2")
	table.insert(groups.CS_cop_C45_R870, "units/pd2_dlc_rvd/characters/ene_la_cop_3/ene_la_cop_3")
	table.insert(groups.CS_cop_C45_R870, "units/pd2_dlc_rvd/characters/ene_la_cop_4/ene_la_cop_4")
end

-- Add the faction.
add_faction(faction_name, chatter, prefixes, groups, difficulty_specific)