local level_id = Global.level_data and Global.level_data.level_id or ""

local gensec_elites = {
	[Idstring("units/payday2/characters/ene_city_swat_1/ene_city_swat_1"):key()] = true,
	[Idstring("units/payday2/characters/ene_city_swat_2/ene_city_swat_2"):key()] = true,
	[Idstring("units/payday2/characters/ene_city_swat_3/ene_city_swat_3"):key()] = true,
	[Idstring("units/payday2/characters/ene_city_swat_r870/ene_city_swat_r870"):key()] = true,
}

Hooks:PostHook(CopBase, "init", "fix_char_tweak", function(self, unit)
	if managers.groupai:state():whisper_mode() then
		if self._tweak_table == "tank" then
			self._char_tweak.allowed_stances = nil
			self._char_tweak.silent_priority_shout = "f37"
			self._char_tweak.has_alarm_pager = true
			self._char_tweak.calls_in = true
			self._char_tweak.suspicious = true
			self._char_tweak.access = "security"
		elseif gensec_elites[unit:name():key()] then
			self._char_tweak.silent_priority_shout = "f37"
			self._char_tweak.has_alarm_pager = true
			self._char_tweak.calls_in = true
			self._char_tweak.suspicious = true
			self._char_tweak.access = "security"
		end
	end
	
	if unit:unit_data().mission_element and unit:unit_data().mission_element._is_elite then
		self._char_tweak.no_arrest = true
		self._char_tweak.access = "security"
	end
end)

local IgnoredCops = {
	[Idstring("units/pd2_dlc_ranc/characters/ene_male_ranc_ranger_01/ene_male_ranc_ranger_01"):key()] = true,
	[Idstring("units/pd2_dlc_ranc/characters/ene_male_ranc_ranger_02/ene_male_ranc_ranger_02"):key()] = true,
}

local SecretService = {
	[Idstring("units/payday2/characters/ene_secret_service_1/ene_secret_service_1"):key()] = true,
	[Idstring("units/payday2/characters/ene_secret_service_2/ene_secret_service_2"):key()] = true,
	[Idstring("units/pd2_dlc_vit/characters/ene_murkywater_secret_service/ene_murkywater_secret_service"):key()] = true,
}

Hooks:PreHook(CopBase, "default_weapon_name", "pd3_default_weapon_name", function(self, selection_name)
	local is_cop = self._unit:base()._tweak_table == "cop"
	local is_guard = self._unit:base()._tweak_table == "security"
	local is_gensec = self._unit:base()._tweak_table == "gensec"
	
	if is_cop and not IgnoredCops[self._unit:name():key()] then
		-- Fix ene_cop_3's weapon to be accurate to the belt
		if self._unit:name():key() == Idstring("units/payday2/characters/ene_cop_3/ene_cop_3"):key() or self._unit:name():key() == Idstring("units/pd2_dlc_rvd/characters/ene_la_cop_3/ene_la_cop_3"):key() then
			self._default_weapon_id = "r870"
		elseif self._unit:name():key() == Idstring("units/payday2/characters/ene_cop_4/ene_cop_4"):key() or self._unit:name():key() == Idstring("units/pd2_dlc_rvd/characters/ene_la_cop_4/ene_la_cop_4"):key() then
			self._default_weapon_id = "mp5"
		else
			if managers.groupai:state():whisper_mode() then
				self._default_weapon_id = "c45"
			else
				local weapons = { "mp5", "m4", "r870" }
				self._default_weapon_id = weapons[math.random(1,#weapons)]
			end
		end
	end
	
	if is_guard then
		if SecretService[self._unit:name():key()] then
			self._default_weapon_id = "mp5"
		else
			self._default_weapon_id = "c45"
		end
	end
	
	if is_gensec then
		local weapons = { "mp5", "c45", "r870" }
		self._default_weapon_id = weapons[math.random(1,#weapons)]
	end
	
	-- Inside lady has no collision and sometimes when she is playing anims the gun sticks out, so give her a c45
	if self._unit:name():key() == Idstring("units/pd2_dlc_red/characters/civ_female_inside_man_1/civ_female_inside_man_1"):key() then
		self._default_weapon_id = "c45"
	end
end)

local RadioGuards = {
	[Idstring("units/payday2/characters/ene_secret_service_1/ene_secret_service_1"):key()] = true,
	[Idstring("units/payday2/characters/ene_secret_service_2/ene_secret_service_2"):key()] = true,
	[Idstring("units/pd2_dlc_vit/characters/ene_murkywater_secret_service/ene_murkywater_secret_service"):key()] = true,
}

Hooks:PostHook(CopBase, "post_init", "_post_spawn_enemy", function(self)
	-- Murkywater Guards have city_swat, so let's make use of that and make their pagers also go off when intimidated
	if self._unit:base()._tweak_table == "city_swat" or self._unit:base()._tweak_table == "security_undominatable" or RadioGuards[self._unit:name():key()] then
		self._unit:unit_data().has_active_radio = true
	end
end)