local level_id = Global.level_data and Global.level_data.level_id or ""
local difficulty = Global.game_settings and Global.game_settings.difficulty or "easy"

local function isDifficultyEqualsOrGreaterThanOverkill()
	if difficulty == "overkill_145" or difficulty == "easy_wish" or difficulty == "overkill_290" or difficulty == "sm_wish" then
		return true
	end
	
	return false
end

local _evaluate_reason_to_surrender = Hooks:GetFunction(CopLogicBase, "_evaluate_reason_to_surrender")
Hooks:OverrideFunction(CopLogicBase, "_evaluate_reason_to_surrender", function(data,my_data,aggressor_unit)
	if data.unit:unit_data().is_civilian then
		return 0
	end
	
	if managers.groupai:state()._task_data.assault.active or managers.groupai:state():get_hunt_mode() or level_id == "hangar_matt" then
		return
	end
	
	return _evaluate_reason_to_surrender(data, my_data, aggressor_unit)
end)

local _can_arrest = Hooks:GetFunction(CopLogicBase, "_can_arrest")
Hooks:OverrideFunction(CopLogicBase, "_can_arrest", function(data)
	if data.unit:unit_data().is_civilian then
		return false
	end
	
	local level_data = tweak_data.levels[level_id]
	
	if level_id == "hangar_matt" 
	or level_data.ai_group_type == "response_type_hell"
	or level_data.ai_group_type == "response_type_bellmead"
	or level_data.ai_group_type == "murkywater"
	or level_data.ai_group_type == "russia"
	or level_data.ai_group_type == "zombie"
	then
		return false
	end
	
	if not Global.game_settings.single_player and isDifficultyEqualsOrGreaterThanOverkill() then
		return false
	end
	
	return _can_arrest(data)
end)

local function GetDetectionProgress()
	if managers.groupai:state():enemy_weapons_hot() then
		return 1
	end
	
	if managers.groupai:state():is_search_mode_active() then
		local detection_rate = 0.5
		
		if managers.groupai:state():get_search_phase() == "alert" then
			detection_rate = 0.9
		end
		
		return detection_rate
	end
	
	return 0
end

local function isCivilian(_tweak_table)
	if string.begins(_tweak_table, "civilian") or _tweak_table == "bank_manager" then
		return true
	end
	
	return false
end

local _create_detected_attention_object_data = Hooks:GetFunction(CopLogicBase, "_create_detected_attention_object_data")
Hooks:OverrideFunction(CopLogicBase, "_create_detected_attention_object_data", function(...)
	local data = _create_detected_attention_object_data(...)
	if data.reaction == "REACT_COMBAT" and not isCivilian(data.unit:base()._tweak_table) then
		data.notice_progress = GetDetectionProgress()
	end
	return data
end)

if StreamHeist or not PD25.settings.shield_tweaks then
	return
end

local mrot_y = mrotation.y
local mvec3_add = mvector3.add
local mvec3_dir = mvector3.direction
local mvec3_dis_sq = mvector3.distance_sq
local mvec3_dot = mvector3.dot
local mvec3_mul = mvector3.multiply
local mvec3_neg = mvector3.negate
local mvec3_set = mvector3.set
local mvec3_set_z = mvector3.set_z
local mvec3_sub = mvector3.subtract
local tmp_vec1 = Vector3()
local tmp_vec2 = Vector3()
local tmp_vec3 = Vector3()

-- Make shield_cover tactics stick closer to their shield tactics providers
Hooks:PreHook(CopLogicBase, "on_new_objective", "sh_on_new_objective", function (data, old_objective)
	if not data.objective or data.objective.type ~= "defend_area" or not data.group or not data.tactics or not data.tactics.shield_cover then
		return
	end

	local shield_unit = old_objective and old_objective.shield_cover_unit
	if not alive(shield_unit) or shield_unit:character_damage():dead() then
		local shield_followers = {}
		for u_key, u_data in pairs(data.group.units) do
			local logic_data = u_data.unit:brain()._logic_data
			if logic_data.tactics then
				if logic_data.tactics.shield_cover then
					local shield_cover_unit = logic_data.objective and logic_data.objective.shield_cover_unit
					if alive(shield_cover_unit) and not shield_cover_unit:character_damage():dead() then
						local shield_key = shield_cover_unit:key()
						shield_followers[shield_key] = (shield_followers[shield_key] or 0) + 1
					end
				elseif logic_data.tactics.shield then
					shield_followers[u_key] = shield_followers[u_key] or 0
				end
			end
		end

		local best_shield_key
		local least_followers = 4
		for u_key, followers in pairs(shield_followers) do
			if followers < least_followers then
				best_shield_key = u_key
				least_followers = followers
			end
		end

		shield_unit = best_shield_key and data.group.units[best_shield_key].unit
	end

	if shield_unit then
		data.objective.type = "follow"
		data.objective.shield_cover_unit = shield_unit
		data.objective.follow_unit = shield_unit
		data.objective.path_data = nil
		data.objective.distance = 300
	end
end)

Hooks:PostHook(CopLogicBase, "on_objective_unit_damaged", "_remove_follow_unit", function(data, unit, attacker_unit)
	if unit:character_damage()._dead then
		data.objective_failed_clbk(data.unit, data.objective)
	end
end)