local level_id = Global.level_data and Global.level_data.level_id or ""
local difficulty = Global.game_settings and Global.game_settings.difficulty or "normal"

Hooks:PostHook(GroupAITweakData, "_init_task_data", "_setup_group_tasks", function(self, difficulty_index)
	self.smoke_grenade_lifetime = 7.5
	self.flash_grenade.timer = 3
	self.besiege.assault.build_duration = 35
	self.besiege.assault.fade_duration = 15
	
	self.besiege.assault.force = {
		6,
		6,
		6
	}
	self.besiege.assault.force_pool = {
		100,
		100,
		100
	}
	self.besiege.recon.force = {
		4,
		4,
		4
	}
	
	-- Let all assault groups retire
	self.besiege.recon.groups = {}
	
	-- Scripted spawns
	self.besiege.assault.groups.map_spawned_persistent = {0,0,0}
	self.besiege.recon.groups.map_spawned_persistent = {0,0,0}
	
	-- Event enemies
	-- These groups are added here so they never retire and do their tasks
	self.besiege.recon.groups.snowman_boss = {0,0,0}
	self.besiege.assault.groups.snowman_boss = {0,0,0}
	self.besiege.recon.groups.piggydozer = {0,0,0}
	self.besiege.assault.groups.piggydozer = {0,0,0}
	self.besiege.assault.groups.single_spooc = {0,0,0}
	
	-- Spawn HRTs between assault breaks
	if difficulty_index <= 2 then
		self.besiege.recon.groups.police_hrt_squad = {0.2,0.2,0.2}
	else
		self.besiege.recon.groups.fbi_hrt_squad = {0.2,0.2,0.2}
	end
end)

-- I am not gonna alter too much here, so you may use other mods that tweak groupai
-- In-fact that's why i opted to use custom factions instead of modifying america
Hooks:PostHook(GroupAITweakData, "_init_unit_categories", "pd3_init_special_limits", function(self, difficulty_index)
	self.flash_grenade.timer = 3
	self.special_unit_spawn_limits.medic = 0
	
	-- Russian HRT
	self.unit_categories.CS_cop_C45_R870.unit_types.russia = {
		Idstring("units/pd2_dlc_mad/characters/ene_akan_cs_cop_ak47_ass/ene_akan_cs_cop_ak47_ass"),
		Idstring("units/pd2_dlc_mad/characters/ene_akan_cs_cop_akmsu_smg/ene_akan_cs_cop_akmsu_smg"),
		Idstring("units/pd2_dlc_mad/characters/ene_akan_cs_cop_r870/ene_akan_cs_cop_r870")
	}
	self.unit_categories.FBI_suit_C45_M4.unit_types.russia = {
		Idstring("units/pd2_dlc_mad/characters/ene_akan_cs_cop_ak47_ass/ene_akan_cs_cop_ak47_ass"),
		Idstring("units/pd2_dlc_mad/characters/ene_akan_cs_cop_akmsu_smg/ene_akan_cs_cop_akmsu_smg")
	}
	self.unit_categories.FBI_suit_M4_MP5.unit_types.russia = {
		Idstring("units/pd2_dlc_mad/characters/ene_akan_cs_cop_akmsu_smg/ene_akan_cs_cop_akmsu_smg"),
		Idstring("units/pd2_dlc_mad/characters/ene_akan_cs_cop_r870/ene_akan_cs_cop_r870")
	}

	-- Murkywater HRT
	self.unit_categories.CS_cop_C45_R870.unit_types.murkywater = {
		Idstring("units/pd2_dlc_bph/characters/ene_murkywater_light/ene_murkywater_light"),
		Idstring("units/pd2_dlc_bph/characters/ene_murkywater_light_r870/ene_murkywater_light_r870")
	}
	self.unit_categories.FBI_suit_C45_M4.unit_types.murkywater = {
		Idstring("units/pd2_dlc_bph/characters/ene_murkywater_light/ene_murkywater_light"),
		Idstring("units/pd2_dlc_bph/characters/ene_murkywater_light_r870/ene_murkywater_light_r870")
	}
	self.unit_categories.FBI_suit_M4_MP5.unit_types.murkywater = {
		Idstring("units/pd2_dlc_bph/characters/ene_murkywater_light/ene_murkywater_light"),
		Idstring("units/pd2_dlc_bph/characters/ene_murkywater_light_r870/ene_murkywater_light_r870")
	}

	-- Federales HRT
	self.unit_categories.CS_cop_C45_R870.unit_types.federales = {
		Idstring("units/pd2_dlc_bex/characters/ene_swat_policia_federale/ene_swat_policia_federale"),
		Idstring("units/pd2_dlc_bex/characters/ene_swat_policia_federale_r870/ene_swat_policia_federale_r870")
	}
	self.unit_categories.FBI_suit_C45_M4.unit_types.federales = {
		Idstring("units/pd2_dlc_bex/characters/ene_swat_policia_federale/ene_swat_policia_federale")
	}
	self.unit_categories.FBI_suit_M4_MP5.unit_types.federales = {
		Idstring("units/pd2_dlc_bex/characters/ene_swat_policia_federale/ene_swat_policia_federale")
	}
end)

-- Handle level specific spawngroups
-- Marshals are locked back to their intended heists to make them a unique enemy again
Hooks:PostHook(GroupAITweakData, "_init_enemy_spawn_groups", "pd3_init_enemy_spawn_groups_level", function (self, difficulty_index)
	-- Setup tactics
	if not table.contains(self._tactics.swat_shotgun_rush, "smoke_grenade") then
		table.insert(self._tactics.swat_shotgun_rush, "smoke_grenade")
	end
	if not table.contains(self._tactics.swat_shotgun_rush, "flash_grenade") then
		table.insert(self._tactics.swat_shotgun_rush, "flash_grenade")
	end
	
	if not table.contains(self._tactics.swat_shotgun_flank, "smoke_grenade") then
		table.insert(self._tactics.swat_shotgun_flank, "smoke_grenade")
	end
	if not table.contains(self._tactics.swat_shotgun_flank, "flash_grenade") then
		table.insert(self._tactics.swat_shotgun_flank, "flash_grenade")
	end
	
	if not table.contains(self._tactics.swat_rifle, "smoke_grenade") then
		table.insert(self._tactics.swat_rifle, "smoke_grenade")
	end
	if not table.contains(self._tactics.swat_rifle, "flash_grenade") then
		table.insert(self._tactics.swat_rifle, "flash_grenade")
	end
	
	if not table.contains(self._tactics.swat_rifle_flank, "smoke_grenade") then
		table.insert(self._tactics.swat_rifle_flank, "smoke_grenade")
	end
	if not table.contains(self._tactics.swat_rifle_flank, "flash_grenade") then
		table.insert(self._tactics.swat_rifle_flank, "flash_grenade")
	end
	
	if level_id == "ranc" then
		self.enemy_spawn_groups.marshal_squad = {
			spawn_cooldown = 60,
			max_nr_simultaneous_groups = 2,
			initial_spawn_delay = 90,
			amount = { 2, 2 },
			spawn = {
				{
					respawn_cooldown = 30,
					amount_min = 1,
					rank = 1,
					freq = 1,
					unit = "marshal_marksman",
					tactics = self._tactics.marshal_marksman
				},
				{
					respawn_cooldown = 30,
					amount_min = 1,
					rank = 1,
					freq = 1,
					unit = "marshal_shield",
					tactics = self._tactics.marshal_shield
				}
			},
			spawn_point_chk_ref = table.list_to_set({
				"tac_shield_wall",
				"tac_shield_wall_ranged",
				"tac_shield_wall_charge"
			})
		}
	elseif level_id == "deep" then
		self.enemy_spawn_groups.marshal_squad = {
			spawn_cooldown = 60,
			max_nr_simultaneous_groups = 2,
			amount = { 2, 2 },
			spawn = {
				{
					respawn_cooldown = 30,
					amount_min = 1,
					rank = 1,
					freq = 1,
					unit = "marshal_marksman",
					tactics = self._tactics.marshal_marksman
				},
				{
					respawn_cooldown = 30,
					amount_min = 1,
					rank = 1,
					freq = 1,
					unit = "marshal_shield",
					tactics = self._tactics.marshal_shield
				}
			},
			spawn_point_chk_ref = table.list_to_set({
				"tac_shield_wall",
				"tac_shield_wall_ranged",
				"tac_shield_wall_charge"
			})
		}
	elseif level_id == "corp" then
		self.enemy_spawn_groups.marshal_squad = {
			spawn_cooldown = 60,
			max_nr_simultaneous_groups = 2,
			initial_spawn_delay = 480,
			amount = { 2, 2 },
			spawn = {
				{
					respawn_cooldown = 30,
					amount_min = 1,
					rank = 1,
					freq = 1,
					unit = "marshal_marksman",
					tactics = self._tactics.marshal_marksman
				},
				{
					respawn_cooldown = 30,
					amount_min = 1,
					rank = 1,
					freq = 1,
					unit = "marshal_shield",
					tactics = self._tactics.marshal_shield
				}
			},
			spawn_point_chk_ref = table.list_to_set({
				"tac_shield_wall",
				"tac_shield_wall_ranged",
				"tac_shield_wall_charge"
			})
		}
	else
		self.enemy_spawn_groups.marshal_squad = nil
	end
	
	self.enemy_spawn_groups.fbi_hrt_squad = {
		amount = {
			2,
			2
		},
		spawn = {
			{
				amount_min = 3,
				freq = 2,
				amount_max = 3,
				rank = 2,
				unit = "FBI_suit_C45_M4",
				tactics = self._tactics.swat_rifle_flank
			},
			{
				amount_min = 1,
				freq = 2,
				amount_max = 1,
				rank = 3,
				unit = "FBI_suit_M4_MP5",
				tactics = self._tactics.swat_rifle_flank
			}
		},
		spawn_point_chk_ref = table.list_to_set({
			"tac_swat_rifle_flank",
			"tac_swat_rifle"
		})
	}
	
	self.enemy_spawn_groups.police_hrt_squad = {
		amount = {
			2,
			2
		},
		spawn = {
			{
				amount_min = 4,
				freq = 2,
				amount_max = 4,
				rank = 2,
				unit = "CS_cop_C45_R870",
				tactics = self._tactics.swat_rifle_flank
			},
		},
		spawn_point_chk_ref = table.list_to_set({
			"tac_swat_rifle_flank",
			"tac_swat_rifle"
		})
	}
end)