local level_id = Global.level_data and Global.level_data.level_id or ""
local difficulty = Global.game_settings and Global.game_settings.difficulty or "normal"

-- Some specific maps we use masked up detection even for casing mode players
local HOSTILE_MAPS = {
	"kosugi",
	"crojob3_night",
	"hox_1",
	"hox_2",
	"hox_3",
	"dark",
	"short1_stage1",
	"short1_stage2",
	"dah",
	"tag",
	"ranc",
	"deep",
	"pex",
	"fex",
	"sand",
	"friend",
	"gallery",
	"welcome_to_the_jungle_1",
	"welcome_to_the_jungle_1_night",
	"welcome_to_the_jungle_2",
	"framing_frame_1",
	"framing_frame_2",
	"framing_frame_3",
	"firestarter_1",
	"firestarter_2",
	"election_day_1",
	"election_day_2",
	"mia_1",
	"mia_2",
	"mia2_new",
	"watchdogs_1",
	"watchdogs_2",
	"watchdogs_2_day",
	"watchdogs_1_night",
	"escape_cafe",
	"escape_park",
	"escape_cafe_day",
	"escape_park_day",
	"escape_street",
	"escape_overpass",
	"escape_garage",
	"escape_overpass_night",
	"arm_fac",
	"arm_par",
	"arm_hcm",
	"arm_cro",
	"arm_und",
	"arm_for",
	"rat",
	"shoutout_raid",
}

local function isDifficultyHigh()
	local high_difficulty = difficulty == "overkill_145" or difficulty == "easy_wish" or difficulty == "overkill_290" or difficulty == "sm_wish"
	return high_difficulty
end

Hooks:PostHook(LevelsTweakData, "init", "_init_levels", function (self)
	-- Enable flashlight state on some night time levels
	self.arm_fac.flashlights_on = true
	self.arm_und.flashlights_on = true
	self.dah.flashlights_on = true
	self.escape_park.flashlights_on = true
	self.firestarter_1.flashlights_on = true
	self.firestarter_2.flashlights_on = true
	self.watchdogs_1_night.flashlights_on = true
	self.watchdogs_2.flashlights_on = true
	self.welcome_to_the_jungle_1_night.flashlights_on = true
	self.kosugi.flashlights_on = true
	self.deep.flashlights_on = true
	
	if table.contains(HOSTILE_MAPS, level_id) then
		self[level_id].is_map_hostile = true
	end
end)

Hooks:PostHook(LevelsTweakData, "init", "_override_groupai", function(self)
	local fbi_levels = {
		["hox_1"] = true,
		["hox_2"] = true,
		["hox_3"] = true,
		["rat"] = true,
		["firestarter_1"] = true,
		["firestarter_2"] = true,
		["firestarter_3"] = true,
		["watchdogs_1"] = true,
		["watchdogs_1_night"] = true,
		["watchdogs_2"] = true,
		["watchdogs_2_day"] = true,
		["watchdogs_3"] = true,
		["alex_1"] = true,
		["alex_2"] = true,
		["alex_3"] = true,
		["mallcrasher"] = true,
		["big"] = true,
		["shoutout_raid"] = true,
		["peta2"] = true,
		["biker_train"] = true,
		["chill_combat"] = true,
		["wwh"] = true,
		["sah"] = true,
		["mus"] = true,
		["pbr2"] = true,
		["crojob2"] = true,
		["crojob3"] = true,
		["trai"] = true,
		["skm_mus"] = true,
		["skm_red2"] = true,
		["skm_run"] = true,
		["skm_watchdogs_stage2"] = true,
		["skm_cas"] = true,
		["skm_big2"] = true,
		["skm_mallcrasher"] = true,
		["skm_arena"] = true,
		["escape_cafe_day"] = true,
		["escape_park_day"] = true,
		["escape_cafe"] = true,
		["escape_park"] = true,
		["escape_street"] = true,
		["escape_overpass"] = true,
		["escape_overpass_night"] = true,
		["escape_garage"] = true,
		["escape_garage"] = true,
		["crojob3_night"] = true,
		["arm_cro"] = true,
		["arm_und"] = true,
		["arm_hcm"] = true,
		["arm_par"] = true,
		["arm_fac"] = true,
		["arm_for"] = true,
		["big"] = true,
		["mia_1"] = true,
		["mia_2"] = true,
		["mia2_new"] = true,
		["welcome_to_the_jungle_1"] = true,
		["welcome_to_the_jungle_2"] = true,
		["welcome_to_the_jungle_1_night"] = true,
		["kenaz"] = true,
		["chill_combat"] = true,
		["spa"] = true,
		["ranc"] = true,
		["trai"] = true,
		["corp"] = true,
	}
	
	local swat_only_levels = {
		["branchbank"] = true,
		["gallery"] = true,
		["rat"] = true,
		["family"] = true,
		["roberts"] = true,
		["nail"] = true,
		["help"] = true,
		["brb"] = true,
		["election_day_1"] = true,
		["election_day_2"] = true,
		["election_day_3"] = true,
		["election_day_3_skip1"] = true,
		["election_day_3_skip2"] = true,
		["framing_frame_1"] = true,
		["framing_frame_2"] = true,
		["framing_frame_3"] = true,
		["jolly"] = true,
		["nightclub"] = true,
		["pines"] = true,
		["moon"] = true,
		["cane"] = true,
		["spa"] = true,
		["red2"] = true,
		["dah"] = true,
		["glace"] = true,
		["run"] = true,
		["nmh"] = true,
		["flat"] = true,
		["dinner"] = true,
		["pal"] = true,
		["man"] = true,
		["friend"] = true,
		["jewelry_store"] = true,
		["ukrainian_job"] = true,
		["four_stores"] = true,
		["pent"] = true,
		["chca"] = true,
		["sand"] = true,
		["chas"] = true,
	}
	
	for level_name,_ in pairs(self) do
		if fbi_levels[level_name] then
			self[level_name].ai_group_type = "response_type_fbi"
		elseif swat_only_levels[level_name] then
			self[level_name].ai_group_type = "response_type_swat"
		elseif level_id == "arena" or level_id == "arm_for" and isDifficultyHigh() then
			self[level_name].ai_group_type = "response_type_gensec"
		elseif level_id == "deep" then
			self[level_name].ai_group_type = "response_type_bellmead"
		elseif level_id == "hangar_matt" then
			self[level_name].ai_group_type = "response_type_hell"
		end
	end
	
	-- Loud tutorials will spawn regular cops
	self.short2_stage1.ai_unit_group_overrides = {
		CS_swat_MP5 = {
			america = {
				Idstring("units/payday2/characters/ene_cop_3/ene_cop_3")
			}
		},
		CS_swat_R870 = {
			america = {
				Idstring("units/payday2/characters/ene_cop_4/ene_cop_4")
			}
		},
		CS_heavy_M4 = {
			america = {
				Idstring("units/payday2/characters/ene_swat_1/ene_swat_1")
			}
		},
		CS_heavy_R870 = {
			america = {
				Idstring("units/payday2/characters/ene_swat_2/ene_swat_2")
			}
		}
	}
	self.short2_stage2b.ai_unit_group_overrides = deep_clone(self.short2_stage1.ai_unit_group_overrides)
	
	self.corp.ai_unit_group_overrides = {
		marshal_shield = {
			america = {
				Idstring("units/pd2_dlc_usm2/characters/ene_male_marshal_shield_1/ene_male_marshal_shield_1")
			}
		},
		marshal_marksman = {
			america = {
				Idstring("units/pd2_dlc_usm1/characters/ene_male_marshal_marksman_1/ene_male_marshal_marksman_1")
			}
		}
	}
	
	self.ranc.ai_unit_group_overrides = deep_clone(self.corp.ai_unit_group_overrides)
	self.trai.ai_unit_group_overrides = deep_clone(self.corp.ai_unit_group_overrides)
end)